import java.util.Random;

DCEL triangulation;

// A reference to any selected point: 
Point2d selectedPoint = null;
DCEL.HalfEdge selectedEdge = null;
int nPoints = 20;

ArrayList<Point2d> S;

int to_x(double coordX) {
  return (int)Math.round(coordX + (width / 2));
}

int to_y(double coordY) {
  return (int)Math.round((height/2) - coordY);
}

void setup() {
  size(800, 600);
  smooth(4);
  background(255);

  triangulateRandomPointSet(473, nPoints);
}

void triangulateRandomPointSet(long seed, int n) {
  S = new ArrayList<Point2d>();
  triangulation = new DCEL();
  
  Random r = new Random(seed);
  for (int i = 0; i < n; i++) {
    S.add(new Point2d(r.nextDouble()*width - (width/2.0), r.nextDouble()*height - (height/2.0)));
  }
  
  triangulation = triangulationIncremental(S);
  selectedEdge = triangulation.outerFace.incidentEdge;
}

void draw() {
  background(255);
  Random rand = new Random(32);
  // Draw faces
  boolean blueFace = false;
  
  //System.out.println("nFaces: " + triangulation.faces.size());
  for (DCEL.Face f : triangulation.faces) {
    //System.out.println("Face size: " + triangulation.getCycleFrom(f.incidentEdge).size());
    //System.out.println(f == triangulation.outerFace);
    if (f != triangulation.outerFace) {
      fill(rand.nextInt(256), rand.nextInt(256), rand.nextInt(256));
      ArrayList<DCEL.HalfEdge> edges = triangulation.getCycleFrom(f.incidentEdge);
      beginShape();
      for (DCEL.HalfEdge he : edges) {
        vertex(to_x(he.origin.coord.x), to_y(he.origin.coord.y));
      }
      endShape(CLOSE);
    }
  }
  
  // Draw edges
  stroke(0);
  for (DCEL.HalfEdge e : triangulation.halfEdges) {
    line(to_x(e.origin.coord.x), to_y(e.origin.coord.y), 
            to_x(e.twin.origin.coord.x), to_y(e.twin.origin.coord.y));
  }
  
  stroke(255, 0, 0);
  strokeWeight(3);
  line(to_x(selectedEdge.origin.coord.x), to_y(selectedEdge.origin.coord.y), 
        to_x(selectedEdge.twin.origin.coord.x), to_y(selectedEdge.twin.origin.coord.y));
  strokeWeight(1);
  
  // Draw vertices
  fill(255);
  stroke(0);
  for (DCEL.Vertex v : triangulation.vertices) {
    ellipse(to_x(v.coord.x), to_y(v.coord.y), 8, 8);
  }
  
  stroke(255, 0, 0);
  strokeWeight(3);
  ellipse(to_x(selectedEdge.origin.coord.x), to_y(selectedEdge.origin.coord.y), 8, 8);
  strokeWeight(1);
  
  // Randomly flip edges
  // Uncomment this code for random walk: 
  //ArrayList<DCEL.HalfEdge> edges = new ArrayList<DCEL.HalfEdge>(triangulation.halfEdges);
  //Random r = new Random(System.currentTimeMillis());
  //edges.get(r.nextInt(edges.size())).flip();
}

void keyReleased() {
  if (key == CODED) {
    if (keyCode == RIGHT) {
      selectedEdge = selectedEdge.next;
    } else if (keyCode == LEFT) {
      selectedEdge = selectedEdge.prev;
    }
  } else if (key == 't') {
    selectedEdge = selectedEdge.twin;
  } else if (key == 'f') {
    selectedEdge.flip();
    selectedEdge = selectedEdge.next;
  } else if (key == 'd') {
    delaunify(triangulation);
    selectedEdge = triangulation.outerFace.incidentEdge;
  } else if (key == 'r') {
    triangulateRandomPointSet(System.currentTimeMillis(), nPoints);
  } else if (key == '+' || key == '=') {
    nPoints ++;
    System.out.println(nPoints);
    triangulateRandomPointSet(System.currentTimeMillis(), nPoints);
  } else if (key == '-' && nPoints > 3) {
    nPoints --;
   triangulateRandomPointSet(System.currentTimeMillis(), nPoints);
  }
}

void keyPressed() {
}